# Copyright 2020 Aeris Communications Inc.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     https://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

import json
import pathlib

# Resolve this user's home directory path
__home_directory = str(pathlib.Path.home())
default_config_filename = __home_directory + "/.aeris_config"

__config = None


def load_config(path=default_config_filename):
    """
    Loads configuration from a path to a file and returns a Python object.
    The file should contain a JSON object.
    Calling this method should only be necessary if you wish to specify your
    own Aeris API endpoints, or if you wish to load the configuration
    generated by the "aeriscli config" command.

    Parameters
    ----------
    path : str
        the path to the configuration file. If omitted, will use the default
        configuration file: ".aeris_config" in your home
        directory. This is the same file created by the "aeriscli config" command.

    Raises an IOError if no such file exists, or a JSONDecodeError if the file was not valid JSON.
    """
    global __config
    with open(path) as my_config_file:
        __config = json.load(my_config_file)
    return __config


def __load_config(path=default_config_filename):
    """
    Loads configuration like 'load_config', but returns an empty object if the
    file is missing.
    """
    global __config
    try:
        with open(path) as my_config_file:
            __config = json.load(my_config_file)
    except IOError:
        __config = {}
    return __config


def __set_default_urls():
    if not __config:
        __load_config()
    __config.setdefault('urls', {})


def get_aerframe_api_url():
    """
    Returns the URL for the AerFrame API as a string.
    """
    __set_default_urls()
    __config['urls'].setdefault('aerframe_ws_api', 'https://api.aerframe.aeris.com')
    return __config['urls']['aerframe_ws_api']


def get_aerframe_longpoll_url():
    """
    Returns the URL for the AerFrame Longpoll service as a string.
    """
    __set_default_urls()
    __config['urls'].setdefault('aerframe_lp_api', 'https://longpoll.aerframe.aeris.com')
    return __config['urls']['aerframe_lp_api']


def get_aeradmin_url():
    """
    Returns the URL for the AerAdmin API as a string.
    """
    __set_default_urls()
    __config['urls'].setdefault('aeradmin_api', 'https://aeradminapi.aeris.com')
    return __config['urls']['aeradmin_api']


def get_aertraffic_url():
    """
    Returns the URL for the AerTraffic API as a string.
    """
    __set_default_urls()
    __config['urls'].setdefault('aertraffic_api', 'https://aertrafficapi.aeris.com')
    return __config['urls']['aertraffic_api']
